import pyZiagn
Test2 = pyZiagn.uniaxialTensileTest(length0=30, Area0=30)
Test2.Title = "Test2"
Test2.importTestData("data/Example2.txt")
Test2.changeUnits()
Test2.cutData("disp", 1.97)
Test2.smoothForce()
Test2.plotForceDisp()
Test2.calcStressEng()
Test2.calcStrainEng()
Test2.plotStressStrainEng()
Test2.calcStressTrue()
Test2.calcStrainTrue()
Test2.plotStressStrainTrue()
Test2.plotStressStrainEngTrue()
Test2.calcElasticModulus(strain0=0.0000, strain1=0.001)
Test2.zeroStrain()
Test2.calcRp02()
Test2.calcLinearLimit()
Test2.calcStressUltimate()
Test2.calcArea()
Test2.plotStressStrainEngAll()
print(Test2.stressUltimate)
print(Test2.stressEng[-1])
print(Test2.stressRP02)
print(Test2.stressLinLimit)
print(Test2.YoungsModulus)